/*****************************************
 * Large Assignment 1: L-Systems
 * Name:    Erin McClure
 * E-mail:  emccluree@unm.edu
 * Course:      CS491, Computational Fabrication
 * Submitted:    09/14/2022
 * This draws an L system repersenting Red apple  
 * ground corver based on the example in class
***********************************************/
class PinkFlowersClass { 
    String[][]rules={{"F","FW"},
        {"T","FFW>>XFW<<XFWX>>FWX<<","FWX"},
        {"X","[+XT][-XT]T[+XT][-XT]T[+XT][-XT][+XT][-XT]","[-XT][+XT]T[-XT][+XT]T[-XT][+XT][-XT][+XT]"},
        {"+","+"},
        {"-","-"},
        {"[","["},
        {"<","<"},
        {">",">"},
        {"W","W"},
        {"]","]"}};


    String computedWord;
    int iterations;
    float size, angle, scaleFactor;

    PinkFlowersClass(float size, float angle, float scaleFactor, String axiom) {
        this.size = size;
        this.angle = angle;
        this.scaleFactor = scaleFactor;
        computedWord = axiom;
        iterations = 0;
    }

    void iteration() {

        char s;
        String newWord = "";
        size = size * scaleFactor;
        angle = angle - 5;
        //loop through computedWord
        for (int i = 0; i < computedWord.length(); i++) {
            //for every symbol
            s = computedWord.charAt(i);
            //loop through rules to find a match
            for (int j = 0; j < rules.length; j++) {
                if (s == rules[j][0].charAt(0)) {
                    if (s == 'Y' || s == 'Z') {
                        int num = (int) (random(3) + 1);
                        //println();
                        newWord = newWord + rules[j][1];
                    } else if (s == 'T') {
                        int num = (int) (random(2) + 1);
                        newWord = newWord + rules[j][num];
                    } else if (s == 'X') {
                        int num = (int) (random(3));
                        //newWord = newWord + rules[j][num];
                        newWord = newWord + rules[j][1];
                    } else {
                        newWord = newWord + rules[j][1];
                    }
                    break;
                }//replacement
            }
        }
        iterations++;
        computedWord = newWord;

        println(iterations +": " +computedWord);
    }

    void drawLSystem(Turtle t) {
        //loop through computedWord and draw each symbol
        char s;
        for (int i = 0; i < computedWord.length(); i++) {
            s = computedWord.charAt(i);
            if (s == 'F') {
                t.forward(-size);
            } else if (s == '+') {
                t.right(angle);
            } else if (s == '-') {
                t.left(angle);
            } else if (s == '[') {
                t.push();
            } else if (s == ']') {
                t.pop();
            } else if (s == '>') {
                t.left(5);
            } else if (s == '<') {
                t.right(5);
            } else if (s == 'W') {
                fill(#074623);
                //ellipse(t.getX(), t.getY(), 10,1);
                //triangle(t.getX(), t.getY(),t.getX()+2.5, t.getY()+2.5,t.getX()-2.5, t.getY()+2.5);
                //tri();
                t.push();
                if (((int) random(2)) % 2 == 0) {
                    t.left(180);
                } else {
                    t.right(270);
                }
                my_arc(size, 20);
                stroke(#078167);
                t.left(70);
                my_arc(size, 20);
                if (((int) random(36) % 3) == 0) {
                    fill(#F00054);
                    flower();
                    stroke(#0F4B3E);
                    fill(#F00054);
                }
                t.pop();

                //ellipse(t.getX(), t.getY(), 10,1);
                //triangle(t.getX(), t.getY(),t.getX()+2.5, t.getY()+2.5,t.getX()-2.5, t.getY()+2.5);
                //tri();
            } else if (s == 'f') {
                t.forward(50);
            } else if (s == '*') {
                t.right(100);
            } else if (s == '/') {
                t.left(100);
            } else if (s == 'Q') {
                starBurst();
            }
        }
    }

    void starBurst() {
        float theta = 360 / 20;
        for (int k = 0; k < 10; k++) {
            t.right(theta);
            t.forward(size / 4);
            t.right(180);
            t.forward(theta);
        }
    }

    void tri() {
        t.push();
        t.penUp();
        t.forward(-size);
        t.right(90);
        t.penDown();
        t.forward(-size / 2);
        t.left(90);
        t.forward(-size / 2);
        t.left(120);
        t.forward(size);
        t.penUp();
        t.pop();
        t.penDown();
    }

    void my_arc(float l, float r) {
        //'''L = θ × (π/180) × r'''
        strokeWeight(1);
        beginShape();
        float theta = (l * 180.0) / (r * PI);
        float theta_inc = theta / l;
        for (int i = 0; i < l; i++) {
            vertex(t.getX(), t.getY());
            t.forward(1);
            t.left(theta_inc);
        }
        endShape(CLOSE);
        fill(#078167);
        strokeWeight(1);
    }

    void flower() {
        float angle = 0;
        float petalLength = random(10) + 5;
        //float angleInc = random(4);
        float startX = t.getX();
        float startY = t.getY();
        for (int i = 0; i < 360; i++) {
            stroke(#D8024D);
            t.forward(15);
            t.right(180);
            t.forward(15);
            t.right(angle);
            angle = 3 + angle;
        }
        fill(#FAD1E0);
        ellipse(t.getX(), t.getY(), 5, 5);
    }
}
